@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad 
  * This file is part of the Xmgf Software.
@@ -27,16 +28,13 @@ typedef struct{
         double FramesPerSlice;
         } t_RotParm;
 
-
-
-
 #ifndef _NO_PROTO 
 void NewAnimation(void);
 void KillAnimation(void);
 void RecordFrame(void);
 void InterPolateFrame(t_RotParm First, t_RotParm Last, double Steps);
 void PlaySingleFrame(void);
-void PlaySlice(int a);
+void PlaySlice(int a,int b);
 void SaveAnimation(char *Filename);
 void LoadAnimation(char *Filename);
 void SetAnimationSpeed(int NewFramesPerSlice);
@@ -58,6 +56,7 @@ extern int  FlagStop;
 extern int  FlagPause;
 extern int  FlagPlay;
 extern int  FlagReverse;
+extern int  FlagAutoRev;
 
 
 
