@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -21,6 +22,8 @@ void             CompressionCleanUp (char *filename);
 int             ImportCmd (char *filename, int ImportType);
 int 		ExtensionTest(char *filename);
 void 		ImportCleanUp(char *filename);
+int 		BlankEnd(char *fullname);
+int 		MakeCompressed(char *fullname,int type);
 #else
 int             CompressionTest ();
 int             CompressionCmd ();
@@ -28,4 +31,6 @@ void             CompressionCleanUp ();
 int             ImportCmd ();
 int 		ExtensionTest();
 void 		ImportCleanUp();
+int 		BlankEnd();
+int 		MakeCompressed();
 #endif
