@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -26,3 +27,4 @@
 #include "BITMAPS/FillBox.xbm"
 #include "BITMAPS/FillTri.xbm"
 #include "BITMAPS/FillCircle.xbm"
+#include "BITMAPS/Sphere.xbm"
