@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -13,11 +14,13 @@ void             InitGlobalVar (void);
 void             InitViewScale (void);
 void             InitFiles (void);
 void             MakeTwoDim (void);
-void 		FindScreenMaxMin(void);
+void 		 FindScreenMaxMin(void);
+void		 InitEmptySet(void);
 #else
 void             InitGlobalVar ();
 void             InitViewScale ();
 void             InitFiles ();
 void             MakeTwoDim ();
-void 		FindScreenMaxMin();
+void 		 FindScreenMaxMin();
+void 		 InitEmptySet();
 #endif
