@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -9,7 +10,7 @@
  */
 
 #ifndef _NO_PROTO
-void             create_Perspective_widget (Widget parent, int Size);
+void             create_Perspective_widget (Widget parent, int XSize,int YSzie);
 #else
 void             create_Perspective_widget ();
 #endif
