@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -10,7 +11,7 @@
 /* 3D Plotter  */
 
 #ifndef _NO_PROTO
-void             create_Stereo_widgets (Widget parent, int Size);
+void             create_Stereo_widgets (Widget parent, int XSize,int YSize);
 #else
 void             create_Stereo_widgets ();
 #endif
