@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -10,6 +11,12 @@
 
 #ifndef _NO_PROTO
 void            SetView (int ViewType);
+void 		create_Ortho_window(void);
+void 		create_Stereo_window(void);
+void 		create_Perspective_window(void);
 #else
 void            SetView ();
+void 		create_Ortho_window();
+void 		create_Stereo_window();
+void 		create_Perspective_window();
 #endif
