@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 /*
  * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
@@ -34,3 +35,4 @@ int             XmErr ();
 #define NOT_IN_THIS_MODE 10
 #define IMPORT_ERROR 11
 #define NO_COMPRESSIONCMD 12
+#define PARM_OUT_OF_DATE 13
