@@ -1,3 +1,4 @@
+/*static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";*/
 
 #define CLASS 1
 #define FAMILY 5
@@ -8,7 +9,7 @@
 #define ADOBE 0
 
 #define COURIER 0
-#define HELVECTICA 1
+#define HELVETICA 1
 #define NEWCENSCH 2
 #define SYMBOL 3
 #define TIMES 4
@@ -28,7 +29,7 @@ extern int FontSlant;
 extern int FontSize;
 extern int FontColour;
 
-extern char *class[CLASS];
+extern char *fontclass[CLASS];
 extern char *family[FAMILY];
 extern char *weight[WEIGHT];
 extern char *slant[SLANT];
