@@ -1,21 +1,20 @@
+/*static const char sccsid[]="@(#)93/12/20 g.matas@ee.surrey.ac.uk 1.4 ConstSet.c";*/
+
+#include "strGM.h"
 #include "LL.h"
 #include "PGLLstand.h"
 
-#ifndef _NO_PROTO 
-char * GMstrdup(char * s);
-#else 
-char * GMstrdup();
-#endif 
-
 /*------------------------------------------------------------------------*/
-t_LLSet 
-ConsLLSet(SetName, GeomStruct, format)  char * SetName; t_LL GeomStruct; char * format;
+t_LLSet ConsLLSet(SetName,GeomStruct,format)
+char * SetName; 
+t_LL GeomStruct; 
+char * format;
 {
   t_LLSet Set;
 
-  Set = (t_LLSet) malloc (sizeof(t_LLSet*));
-  Set->id = GMstrdup(SetName);
-  Set->format = GMstrdup(format);
+  Set = (t_LLSet) malloc (sizeof(*Set));
+  Set->id = DupStr(SetName);
+  Set->format = DupStr(format);
   Set->GeomStruct = GeomStruct;
   Set->Attributes = ConsLL();
   Set->data = ConsLL();
@@ -23,8 +22,8 @@ ConsLLSet(SetName, GeomStruct, format)  char * SetName; t_LL GeomStruct; char *
   return Set;
 }
 
-void 
-DestLLSet(Set)  t_LLSet Set;
+void DestLLSet(Set)
+t_LLSet Set;
 {
  
   DestLL(Set->data);
