@@ -1,7 +1,15 @@
 #ifndef linkLL_H
 #define  linkLL_H
-/* ------------------ basic link stuff ----------------------------------   */
-/* Operations on links; to be used by the initiated only !!                 */
+/* ------------------ basic link stuff ----------------------------------*/
+/*  original link-node code from
+
+    Anasazi Linked List Utility Routines - 1.15
+    by Duane Morse     e-mail: duane@anasaz (or ... asuvax!anasaz!duane)
+
+    modified end extended by
+        G. Matas   (g.matas@ee.surrey.ac.uk)
+*/
+
 
 #define L_BASIC_MACRO 1         /* implement l_nextl, l_prevl as macros */
 
@@ -31,13 +39,15 @@ void l_unlink();
 
 #define ForeachLink_M(head,link)\
      for(link=l_nextl(head); head != link; link=l_nextl(link))
+#define ForeachLinkBack_M(head,link)\
+     for(link=l_prevl(head); head != link; link=l_prevl(link))
 
 /*--------------------------------------------------------------------------*/
 
 /* conversion link - LL */
-#define elm2link(el)  ((l_list *)(el) - 1)
-#define link2elm(el)  ((void *)  ((el) + 1))
-#define list2link(list) (&(list->links))
+#define elm2link(el)  ((l_list*)((union_l_list *)(el) - 1))
+#define link2elm(li)  ((void *) ((union_l_list *)(li) + 1))
+#define list2link(list) (&(list->links.u.ll))
 
 #endif
 
