@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
  * README file.
  *
@@ -16,6 +17,7 @@
 #include "Pixmaps.h"
 #include "3Droutines.h"
 #include "Parse.h"
+#include "XmgfFont.h"
 
 FILE           *FrameFilePtr;
 int 		OutFrame;
@@ -82,10 +84,14 @@ int ShadedVal;
 {
 	int i;
 	int FrameColour = -1;
+	int HaloedFrameColour=-1;
 	int P = DefaultPointSize;
 	int FrameFill = 0;
+	int si;
+	char FontName[30];
 
 	FrameColour = SetFrameColour(Color);
+	HaloedFrameColour = SetFrameColour(HaloedColor);
 	fill =  DetermineFillType(fill,num,PtType,FillStyle,BaseType);
 
 	if (Color < 64) {
@@ -230,6 +236,23 @@ int ShadedVal;
 		fprintf(FrameFilePtr, "> # end ellipse\n");
 		break;
 		}
+            case FILLEDSPHERE:{
+		for(i=9;i>1;i--){
+                si = P/9*i;
+		fprintf(FrameFilePtr, "<Ellipse\n");
+                fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
+                fprintf(FrameFilePtr, "\t<Pen %d>\n",abs(i-9));
+                fprintf(FrameFilePtr, "\t<PenWidth %d>\n",0);
+                fprintf(FrameFilePtr, "\t<Fill %d>\n",abs(i-9));
+                fprintf(FrameFilePtr, "\t<Separation %d>\n",FrameColour);
+                fprintf(FrameFilePtr, "\t\t<BRect  %f\" %f\" %f\" %f\">\n",
+                ((float)Pts[0].x-(si)+(si/2))/FmSc ,((float)Pts[0].y-(si)+(si/2))/FmSc, ((float) 2*si)/FmSc,((float) 2*si)/FmSc);
+		/* Old size too small 5/1/94 */
+/*                ((float)Pts[0].x-(si/2)+(si/4))/FmSc ,((float)Pts[0].y-(si/2)+(si/4))/FmSc, ((float) si)/FmSc,((float) si)/FmSc);*/
+                fprintf(FrameFilePtr, "> # end ellipse\n");
+		}
+                break;
+                }
 	    case FILLEDOPAQUE:{/* Polygon and PolyLine */
 		fprintf(FrameFilePtr, "<Polygon\n");
 		fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
@@ -256,6 +279,34 @@ int ShadedVal;
 		fprintf(FrameFilePtr, "> # end of Polygon\n");
 		break;
 	    }
+	    case HALOEDLINES:{/* Polygon and PolyLine */
+/*		FrameColour = 1;*/
+		if (num>2) {fprintf(FrameFilePtr, "<Polygon\n"); }
+		else {fprintf(FrameFilePtr, "<PolyLine\n"); }
+		fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
+		fprintf(FrameFilePtr, "\t<Fill 15>\n");
+		fprintf(FrameFilePtr, "\t<Pen 7>\n");
+		fprintf(FrameFilePtr, "\t<PenWidth %d>\n",LineWidth+HALSIZE);
+		fprintf(FrameFilePtr, "\t<Separation %d>\n",HaloedFrameColour);
+		fprintf(FrameFilePtr, "\t\t<NumPoints %d>\n",num);
+		for (i = 0; i < num; i++) {
+		fprintf(FrameFilePtr, "\t\t<Point %f\" %f\">\n",(float)Pts[i].x/FmSc,(float)Pts[i].y/FmSc);}
+		fprintf(FrameFilePtr, "> # end of Polygon\n");
+		FrameColour = 0;
+		if (num>2) {fprintf(FrameFilePtr, "<Polygon\n"); }
+		else {fprintf(FrameFilePtr, "<PolyLine\n"); }
+                fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
+                fprintf(FrameFilePtr, "\t<Fill 15>\n");
+                fprintf(FrameFilePtr, "\t<Pen 0>\n");
+                fprintf(FrameFilePtr, "\t<PenWidth %d>\n",LineWidth);
+                fprintf(FrameFilePtr, "\t<Separation %d>\n",FrameColour);
+                fprintf(FrameFilePtr, "\t\t<NumPoints %d>\n",num);
+                for (i = 0; i < num; i++) {
+                fprintf(FrameFilePtr, "\t\t<Point %f\" %f\">\n",(float)Pts[i].x/FmSc,(float)Pts[i].y/FmSc);}
+                fprintf(FrameFilePtr, "> # end of Polygon\n");
+
+		break;
+	    }
 	    case FILLEDHIDDENLINE:{
 		if(DitherFill){
 		FrameFill = (int) (((double) ShadedVal / 64.0) * 7.0);
@@ -275,10 +326,8 @@ int ShadedVal;
 		}
 	    case WHITELINES:{
 		FrameColour = 1;
-		if (num>2) {fprintf(FrameFilePtr, "<Polygon\n");
-		}
-		else {fprintf(FrameFilePtr, "<PolyLine\n");
-		}
+		if (num>2) {fprintf(FrameFilePtr, "<Polygon\n"); }
+		else {fprintf(FrameFilePtr, "<PolyLine\n"); }
 		fprintf(FrameFilePtr, "\t<Fill 15>\n");
 		fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
 		
@@ -344,6 +393,14 @@ int ShadedVal;
                 break;
 		}
 	    case TEXTLABEL:{  fprintf(FrameFilePtr, "<TextLine\n");
+		  switch(FontFamily){
+         	  case COURIER: strcpy(FontName,"Courier");break;
+         	  case NEWCENSCH: strcpy(FontName,"NewCenturySchlbk");break;
+                  case SYMBOL: strcpy(FontName,"Symbol");break;
+                  case HELVETICA: strcpy(FontName,"Helvetica");break;
+                  case TIMES: strcpy(FontName,"Times");break;
+                  default: strcpy(FontName,"Helvetica");break;
+		  }
                 fprintf(FrameFilePtr, "\t<GroupID\t%d>\n",GROUPID);
                 fprintf(FrameFilePtr, "\t\t<TLOrigin %f\" %f\">\n",((float)Pts[0].x-
 P)/ FmSc ,((float)Pts[0].y-P)/FmSc);
@@ -351,9 +408,9 @@ P)/ FmSc ,((float)Pts[0].y-P)/FmSc);
                 fprintf(FrameFilePtr, "\t<Angle 0>\n");
                 fprintf(FrameFilePtr, "\t<Font\n");
                 fprintf(FrameFilePtr, "\t<FTag `'>\n");
-                fprintf(FrameFilePtr, "\t<FFamily `Times'>\n");
+                fprintf(FrameFilePtr, "\t<FFamily `%s'>\n",FontName);
                 fprintf(FrameFilePtr, "\t<FWeight `Regular'>\n");
-                fprintf(FrameFilePtr, "\t<FSize  8.0 pt>\n");
+                fprintf(FrameFilePtr, "\t<FSize  %d.0 pt>\n",FontSize);
                 fprintf(FrameFilePtr, "\t<FUnderline No >\n\t<FOverline No >\n");
                 fprintf(FrameFilePtr, "\t<FStrike No >\n\t<FSupScript No >\n");
                 fprintf(FrameFilePtr, "\t<FSubScript No >\n\t<FChangeBar No >\n");
