@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
  * README file.
  *
@@ -11,6 +12,8 @@
 #include <malloc.h>
 #include <assert.h>
 
+#include "XmgfMalloc.h"
+
 #include <Xm/Xm.h>
 #include <MotifUtils.h>
 
@@ -32,7 +35,7 @@ MotStr2Str(MotifString)
 XmString *MotifString;
 {
 	char *CharString;
-	CharString = (char *) malloc(MAX_STR_LEN * sizeof(char));
+	CharString = (char *) XmgfMalloc(MAX_STR_LEN * sizeof(char));
 	assert(CharString);
 	XmStringGetLtoR(*MotifString, XmSTRING_DEFAULT_CHARSET, &CharString);
 	return CharString;
