@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
  * README file.
  *
@@ -7,8 +8,10 @@
  * University of Surrey. Guildford, Surrey. England. Phone: +44 483 300800
  * Ext 2753 Email: P.Hoad@ee.surrey.ac.uk
  */
+#include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#include "XmgfMalloc.h"
 
 #include "XmgfStruct.h"
 #include "XmgfGlobal.h"
@@ -41,6 +44,7 @@
 #include "Read3DSet.h"
 #include "Animator.h"
 #include "XmgfHelp.h"
+#include "Compression.h"
 
 char           *PGMDIR;
 char           *PGMFIL;
@@ -76,6 +80,10 @@ char           *MIFDIR;
 char           *MIFFIL;
 char 	       *ANIMDIR;
 char 	       *ANIMFIL;
+char           *PCXDIR;
+char           *PCXFIL;
+char           *PICTDIR;
+char           *PICTFIL;
 char           *CurrentFile;
 
 
@@ -88,25 +96,69 @@ char           *CurrentFile;
 #define MONO(rd,gn,bl) (((rd)*11 + (gn)*16 + (bl)*5) >> 5)	/* .33R+ .5G+ .17B */
 
 #define MAX_CELLS  256
+#define MAX_COUNT  128
+
+typedef long int count_int;
 
 typedef struct {
 	XImage *ximage;
 	int numcells;
 	int red[MAX_CELLS], green[MAX_CELLS], blue[MAX_CELLS];
-	unsigned char used[MAX_CELLS];
+	byte used[MAX_CELLS];
 } imageInfo;
 
 typedef struct {
 	XImage *ximage;
 	int numcells;
-	unsigned char red[MAX_CELLS], green[MAX_CELLS], blue[MAX_CELLS];
-	unsigned char used[MAX_CELLS];
+	byte red[MAX_CELLS], green[MAX_CELLS], blue[MAX_CELLS];
+	byte used[MAX_CELLS];
 } RASimage;
 
-typedef long int count_int;
-typedef unsigned char byte;
 static byte pc2nc[256], r1[256], g1[256], b1[256];
 int format = ZPixmap;
+Widget w_Compression;
+
+#define HEADER_SIZE             512
+#define RUN_THRESH              3
+#define MAX_RUN                 128             /* 0xff = 2, 0xfe = 3, etc */
+#define MAX_COUNT               128             /* 0x00 = 1, 0x01 = 2, etc */
+
+#define PICT_picVersion         0x11
+#define PICT_headerOp           0x0C00
+#define PICT_clipRgn            0x01
+#define PICT_BitsRect           0x90
+#define PICT_PackBitsRect       0x98
+#define PICT_EndOfPicture       0xFF
+
+
+
+#ifndef _NO_PROTO 
+void FindWindowSize(Window win,int WinWidth,int WinHeight);
+unsigned int GetDrawWidth(Window Win);
+unsigned int GetDrawHeight(Window Win);
+void WritePict(FILE *fp);
+void WritePictHeader(FILE *fp,int width,int height);
+static void putFill(FILE *fd, int n);
+static void putShort(FILE *fd, int n);
+static void putLong(FILE *fd, long n);
+static void putFixed(FILE *fd, int n,int frac);
+static void putRect(FILE *fd, int x1, int x2, int y1, int y2);
+static int putRow(FILE *fd, int row, int height, byte *rowpixels,char *packed);
+#else 
+void FindWindowSize();
+unsigned int GetDrawWidth();
+unsigned int GetDrawHeight();
+void WritePict();
+void WritePictHeader();
+void WritePictData();
+static void putFill();
+static void putShort();
+static void putLong();
+static void putFixed();
+static void putRect();
+static int putRow();
+#endif 
+
 
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
@@ -128,10 +180,7 @@ Widget w;
 caddr_t client_data;
 caddr_t call_data;
 {
-	if (theDepth < 8) {
-		XmErr(MONO_NOSAVE);
-		return ;
-	}
+	if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
 	XmFileSelect("Saving XBM File", SAVE_XBM_FILE, XBMFIL, "Out.xbm", XBMDIR);
 	return ;
 }
@@ -156,14 +205,38 @@ Widget w;
 caddr_t client_data;
 caddr_t call_data;
 {
-	if (theDepth < 8) {
-		XmErr(MONO_NOSAVE);
-		return ;
-	}
+	if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
 	XmFileSelect("Saving PPM File", SAVE_PPM_FILE, PPMFIL, "Out.ppm", PPMDIR);
 	return ;
 }
 
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+void
+SavePCXCB(w, client_data, call_data)
+Widget w;
+caddr_t client_data;
+caddr_t call_data;
+{
+        if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
+        XmFileSelect("Saving PCX File", SAVE_PCX_FILE, PCXFIL, "Out.pcx", PCXDIR);
+        return ;
+}
+
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+void
+SavePICTCB(w, client_data, call_data)
+Widget w;
+caddr_t client_data;
+caddr_t call_data;
+{
+        if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
+        XmFileSelect("Saving PICT File", SAVE_PICT_FILE, PICTFIL, "Out.pict", PICTDIR);
+        return ;
+}
+
+
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 void
@@ -172,10 +245,7 @@ Widget w;
 caddr_t client_data;
 caddr_t call_data;
 {
-	if (theDepth < 8) {
-		XmErr(MONO_NOSAVE);
-		return ;
-	}
+	if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
 	XmFileSelect("Saving GIF File", SAVE_GIF_FILE, GIFFIL, "Out.gif", GIFDIR);
 	return ;
 }
@@ -191,10 +261,7 @@ caddr_t call_data;
 	int ColourRequest = (int) client_data;
 	int FileType;
 
-	if (theDepth < 8) {
-		XmErr(MONO_NOSAVE);
-		return ;
-	}
+	if (theDepth < 8) { XmErr(MONO_NOSAVE); return ; }
 	switch (ColourRequest) {
 	    case SAVE_COLOUR:
 		FileType = SAVE_RASCOL_FILE;
@@ -254,6 +321,7 @@ Window GraphicArea;
 	BusyCursor();
 	out_file = fopen(fname, "w");
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y, (unsigned int)width, (unsigned int)height, AllPlanes, format);
 	image.ximage = ximage;
 
@@ -272,7 +340,7 @@ Window GraphicArea;
 	BusyCursor();
 
 	/* Trasnfer Pixmap to PBM file */
-	newBuffer = (int *) malloc(width * height * sizeof(int));
+	newBuffer = (int *) XmgfMalloc(width * height * sizeof(int));
 	assert(newBuffer);
 	BuffCount = 0;
 	BusyCursor();
@@ -350,6 +418,7 @@ Window GraphicArea;
 	if (foo)
 		*foo = '\0';	/* truncated name at first '.' */
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y, (unsigned int)width,(unsigned int) height, AllPlanes, format);
 	image.ximage = ximage;
 
@@ -368,7 +437,7 @@ Window GraphicArea;
 	}
 	BusyCursor();
 	/* Trasnfer Pixmap to XBM file */
-	newBuffer = (int *) malloc(width * height * sizeof(int));
+	newBuffer = (int *) XmgfMalloc(width * height * sizeof(int));
 	assert(newBuffer);
 	BuffCount = 0;
 	for (i = 0; i < width * height; i++) {
@@ -390,7 +459,7 @@ Window GraphicArea;
 				k |= 0x80;
 			bit++;
 			if (bit == 8) {
-				fprintf(out_file, "0x%02x", (unsigned char) ~k);
+				fprintf(out_file, "0x%02x", (byte) ~k);
 				nbytes--;
 				len += 4;
 				if (nbytes) {
@@ -407,7 +476,7 @@ Window GraphicArea;
 
 		if (bit) {
 			k = k >> (8 - bit);
-			fprintf(out_file, "0x%02x", (unsigned char) ~k);
+			fprintf(out_file, "0x%02x", (byte) ~k);
 			nbytes--;
 			len += 4;
 			if (nbytes) {
@@ -465,6 +534,7 @@ caddr_t call_data;
 {
 	Arg args[5];
 	int Orientation = 0;
+        int CompressMe=0;
 	char *DirectoryName;
 	char *FilterName;
 	XmString MotifString, FilterString;
@@ -478,35 +548,41 @@ caddr_t call_data;
 	XtGetValues(FileSelector, args, 1);
 	XtSetArg(args[0], XmNpattern, &FilterString);
 	XtGetValues(FileSelector, args, 1);
-	DirectoryName = (char *) malloc(256 * sizeof(char));
+	CompressMe = (XmToggleButtonGetState(w_Compression) ? 1:0);
+	DirectoryName = (char *) XmgfMalloc(256 * sizeof(char));
 	assert(DirectoryName);
-	FilterName = (char *) malloc(256 * sizeof(char));
+	FilterName = (char *) XmgfMalloc(256 * sizeof(char));
 	assert(FilterName);
 	XmStringGetLtoR(MotifString, charset, &DirectoryName);
 	XmStringGetLtoR(FilterString, charset, &FilterName);
 	LoadOpen = 1;
 	BusyCursor();
-	if (Selectedfilename != NULL) {
+	if(BlankEnd(Selectedfilename)) XmErr(EMPTY_STRING);
+	else if (Selectedfilename != NULL) {
 		switch ((int) client_data) {
 		    case SAVE_PGM_FILE:
-			OutputPGM(Selectedfilename, 0, 0, 512, 512, w_Orth);
+			OutputPGM(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth);
 			strcpy(PGMDIR, DirectoryName);
 			strcpy(PGMFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_XBM_FILE:
-			OutputXBM(Selectedfilename, 0, 0, 512, 512, w_Orth);
+			OutputXBM(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth);
 			strcpy(XBMDIR, DirectoryName);
 			strcpy(XBMFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_PBM_FILE:
-			OutputPBM(Selectedfilename, 0, 0, 512, 512, w_Orth);
+			OutputPBM(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth);
 			strcpy(PBMDIR, DirectoryName);
 			strcpy(PBMFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_PARM_FILE:
 			OutputPARM(Selectedfilename);
 			strcpy(PARMDIR, DirectoryName);
 			strcpy(PARMFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case LOAD_PARM_FILE:
 			InputPARM(Selectedfilename);
@@ -517,56 +593,81 @@ caddr_t call_data;
 			OutputFIG(Selectedfilename, SAVE_GREY);
 			strcpy(FIGDIR, DirectoryName);
 			strcpy(FIGFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_FIGCOL_FILE:
 			OutputFIG(Selectedfilename, SAVE_COLOUR);
 			strcpy(FIGDIR, DirectoryName);
 			strcpy(FIGFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_EPS_FILE:
 			OutputPS(Selectedfilename, SAVE_COLOUR, Orientation);
 			strcpy(EPSDIR, DirectoryName);
 			strcpy(EPSFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_PS_FILE:
 			OutputPS(Selectedfilename, SAVE_GREY, Orientation);
 			strcpy(POSTDIR, DirectoryName);
 			strcpy(POSTFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_PSCOL_FILE:
 			OutputPS(Selectedfilename, SAVE_COLOUR, Orientation);
 			strcpy(POSTDIR, DirectoryName);
 			strcpy(POSTFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_MIF_FILE:
 			OutputFRAME(Selectedfilename, 0);
 			strcpy(MIFDIR, DirectoryName);
 			strcpy(MIFFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_RAY_FILE:
 			OutputRAY(Selectedfilename);
 			strcpy(RAYDIR, DirectoryName);
 			strcpy(RAYFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_GIF_FILE:
-			OutputGIF(Selectedfilename, 0, 0, 512, 512, w_Orth);
+			OutputGIF(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth);
 			strcpy(GIFDIR, DirectoryName);
 			strcpy(GIFFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_RAS_FILE:
-			OutputRAS(Selectedfilename, 0, 0, 512, 512, w_Orth, SAVE_GREY);
+			OutputRAS(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), 
+GetDrawHeight(w_Orth), w_Orth,SAVE_GREY);
 			strcpy(RASDIR, DirectoryName);
 			strcpy(RASFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_RASCOL_FILE:
-			OutputRAS(Selectedfilename, 0, 0, 512, 512, w_Orth, SAVE_COLOUR);
+			OutputRAS(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), 
+GetDrawHeight(w_Orth), w_Orth, SAVE_COLOUR);
 			strcpy(RASDIR, DirectoryName);
 			strcpy(RASFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case SAVE_PPM_FILE:
-			OutputPPM(Selectedfilename, 0, 0, 512, 512, w_Orth);
+			OutputPPM(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth);
 			strcpy(PPMDIR, DirectoryName);
 			strcpy(PPMFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
+			break;
+		    case SAVE_PCX_FILE:
+			OutputPPM(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth); 
+			strcpy(PCXDIR, DirectoryName);
+			strcpy(PCXFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
+			break;
+		    case SAVE_PICT_FILE:
+			OutputPICT(Selectedfilename, 0, 0, GetDrawWidth(w_Orth), GetDrawHeight(w_Orth), w_Orth); 
+			strcpy(PICTDIR, DirectoryName);
+			strcpy(PICTFIL, FilterName);
+			if (CompressMe) MakeCompressed(Selectedfilename,0);
 			break;
 		    case IMPORT_OFF_FILE:
 			ImportFile(Selectedfilename, IMPORT_OFF_FILE);
@@ -654,18 +755,19 @@ caddr_t call_data;
 				SaveAnimation(Selectedfilename);
 				strcpy(ANIMDIR, DirectoryName);
 				strcpy(ANIMDIR, FilterName);
+				if (CompressMe) MakeCompressed(Selectedfilename,0);
 				break;
 
 			}
 
 		}
 	}			/* if */
+	else {XmErr(EMPTY_STRING);}
 	DoneCursor();
 	LoadOpen = 0;
 	free(DirectoryName);
 	free(FilterName);
 	FileSelector = 0;
-
 	return 0;
 }
 
@@ -700,8 +802,8 @@ char *DirName;
 {
 	Widget w_FS_RC;
 	Widget w_Label;
-	Widget w_Compression;
 	Arg args[10];
+	unsigned int n;
 
 	XtSetArg(args[0], XtNwidth, 310);
 	XtSetArg(args[1], XmNautoUnmanage, True);
@@ -718,10 +820,11 @@ char *DirName;
 	w_Label = XtCreateManagedWidget("w_Label",
 		xmLabelWidgetClass, w_FS_RC, args, 1);
 
-	XtSetArg(args[0], XmNlabelString, Str2MotStr("Compress on Save"));
-	XtSetArg(args[1], XmNsensitive, FALSE);
+	n=0;
+	XtSetArg(args[n], XmNlabelString, Str2MotStr("Compress on Save"));n++;
+/*	XtSetArg(args[n], XmNsensitive, FALSE);n++;*/
 	w_Compression = XtCreateManagedWidget("w_Compression",
-		xmToggleButtonWidgetClass, w_FS_RC, args, 2);
+		xmToggleButtonWidgetClass, w_FS_RC, args, n);
 
 	return (w_FileSelectorDia);
 }
@@ -730,8 +833,6 @@ char *DirName;
  * XV2.21 Derived from Output Handling Routines Author:    John Bradley,
  * University of Pennsylvania (bradley@cis.upenn.edu)
  */
-
-
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 void
@@ -750,7 +851,7 @@ Window GraphicArea;
 	Colormap cmap;
 	int ArrayIndex;
 	XColor colors[MAX_CELLS];
-	unsigned char *newBuffer;
+	byte *newBuffer;
 	XImage *ximage;
 
 	/* Open Output File */
@@ -758,6 +859,7 @@ Window GraphicArea;
 	BusyCursor();
 	out_file = fopen(fname, "w");
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y,(unsigned int) width,(unsigned int) height, AllPlanes, format);
 	image.ximage = ximage;
 
@@ -777,15 +879,15 @@ Window GraphicArea;
 	BusyCursor();
 
 	/* Trasnfer Pixmap to PPM file */
-	newBuffer = (unsigned char *) malloc(width * height * 3 * sizeof(unsigned char));
+	newBuffer = (byte *) XmgfMalloc(width * height * 3 * sizeof(byte));
 	assert(newBuffer);
 	BuffCount = 0;
 	BusyCursor();
 	for (i = 0; i < width * height; i++) {
 		ArrayIndex = image.ximage->data[i];
-		newBuffer[BuffCount++] = (unsigned char) image.red[ArrayIndex];
-		newBuffer[BuffCount++] = (unsigned char) image.green[ArrayIndex];
-		newBuffer[BuffCount++] = (unsigned char) image.blue[ArrayIndex];
+		newBuffer[BuffCount++] = (byte) image.red[ArrayIndex];
+		newBuffer[BuffCount++] = (byte) image.green[ArrayIndex];
+		newBuffer[BuffCount++] = (byte) image.blue[ArrayIndex];
 	}
 	BusyCursor();
 	fprintf(out_file, "P6\n%d %d\n%d\n", (width - x), (height - y), 255);
@@ -820,7 +922,7 @@ Window GraphicArea;
 	int ArrayIndex;
 	double Red, Blue, Green;
 	XColor colors[MAX_CELLS];
-	unsigned char *newBuffer;
+	byte *newBuffer;
 	XImage *ximage;
 
 	/* Open Output File */
@@ -828,6 +930,7 @@ Window GraphicArea;
 	BusyCursor();
 	out_file = fopen(fname, "w");
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y,(unsigned int) width,(unsigned int) height, AllPlanes, format);
 	image.ximage = ximage;
 
@@ -847,7 +950,7 @@ Window GraphicArea;
 	BusyCursor();
 
 	/* Trasnfer Pixmap to PGM file */
-	newBuffer = (unsigned char *) malloc(width * height * sizeof(unsigned char));
+	newBuffer = (byte *) XmgfMalloc(width * height * sizeof(byte));
 	assert(newBuffer);
 	BuffCount = 0;
 	for (i = 0; i < width * height; i++) {
@@ -855,7 +958,7 @@ Window GraphicArea;
 		Red = (double) image.red[ArrayIndex];
 		Green = (double) image.green[ArrayIndex];
 		Blue = (double) image.blue[ArrayIndex];
-		newBuffer[BuffCount++] = (unsigned char)
+		newBuffer[BuffCount++] = (byte)
 			(0.299 * Red + 0.587 * Green + 0.114 * Blue);
 	}
 	BusyCursor();
@@ -890,10 +993,10 @@ int Col;
 	Colormap cmap;
 	int ArrayIndex;
 	XColor colors[MAX_CELLS];
-	unsigned char *newBuffer;
+	byte *newBuffer;
 	XImage *ximage;
 	struct rasterfile SunRaster;
-	unsigned char *SingleLine;
+	byte *SingleLine;
 	int ScreenDepth;
 	int LineLength;
 
@@ -902,6 +1005,7 @@ int Col;
 	BusyCursor();
 	out_file = fopen(fname, "w");
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y,(unsigned int) width,(unsigned int) height, AllPlanes, format);
 	image.ximage = ximage;
 
@@ -920,17 +1024,17 @@ int Col;
 
 	BusyCursor();
 	/* Trasnfer Pixmap to PPM file */
-	newBuffer = (unsigned char *) malloc(width * height * 3 * sizeof(unsigned char));
+	newBuffer = (byte *) XmgfMalloc(width * height * 3 * sizeof(byte));
 	assert(newBuffer);
 	BuffCount = 0;
 	for (i = 0; i < width * height; i++) {
 		ArrayIndex = image.ximage->data[i];
-		newBuffer[BuffCount++] = (unsigned char) ArrayIndex;
+		newBuffer[BuffCount++] = (byte) ArrayIndex;
 	}
 	BusyCursor();
 
 	LineLength = width;
-	SingleLine = (unsigned char *) malloc(LineLength);
+	SingleLine = (byte *) XmgfMalloc(LineLength);
 	assert(SingleLine);
 	ScreenDepth = 8;
 	SunRaster.ras_magic = RAS_MAGIC;
@@ -952,9 +1056,9 @@ int Col;
 	write_sun_long(SunRaster.ras_maplength, out_file);
 
 	/* write the colormap */
-	fwrite(image.red, sizeof(unsigned char), ncolors, out_file);
-	fwrite(image.green, sizeof(unsigned char), ncolors, out_file);
-	fwrite(image.blue, sizeof(unsigned char), ncolors, out_file);
+	fwrite(image.red, sizeof(byte), ncolors, out_file);
+	fwrite(image.green, sizeof(byte), ncolors, out_file);
+	fwrite(image.blue, sizeof(byte), ncolors, out_file);
 
 	/* write the image */
 	BusyCursor();
@@ -962,7 +1066,7 @@ int Col;
 	for (y = 0; y < height; y++) {
 		memcpy(SingleLine, newBuffer + y * width, width);
 
-		if (fwrite(SingleLine, sizeof(unsigned char),
+		if (fwrite(SingleLine, sizeof(byte),
 				LineLength, out_file) != LineLength) {
 			free(SingleLine);
 
@@ -1026,7 +1130,7 @@ Window GraphicArea;
 {
 	imageInfo image;
 	FILE *out_file;
-	unsigned char *Red, *Blue, *Green;
+	byte *Red, *Blue, *Green;
 	int i, ncolors;
 	Colormap cmap;
 	XColor colors[MAX_CELLS];
@@ -1037,17 +1141,18 @@ Window GraphicArea;
 	BusyCursor();
 	out_file = fopen(fname, "w");
 	/* Grap window to Pixmap */
+	width-=2;
 	ximage = XGetImage(theDisplay, GraphicArea, x, y,(unsigned int) width,(unsigned int) height, AllPlanes, format);
 	image.ximage = ximage;
 
 	cmap = DefaultColormap(theDisplay, theScreen);
 	ncolors = DisplayCells(theDisplay, theScreen);
 
-	Red = (unsigned char *) malloc(ncolors * sizeof(unsigned char));
+	Red = (byte *) XmgfMalloc(ncolors * sizeof(byte));
 	assert(Red);
-	Green = (unsigned char *) malloc(ncolors * sizeof(unsigned char));
+	Green = (byte *) XmgfMalloc(ncolors * sizeof(byte));
 	assert(Green);
-	Blue = (unsigned char *) malloc(ncolors * sizeof(unsigned char));
+	Blue = (byte *) XmgfMalloc(ncolors * sizeof(byte));
 	assert(Blue);
 	for (i = 0; i < ncolors; i++)
 		colors[i].pixel = i;
@@ -1057,13 +1162,13 @@ Window GraphicArea;
 		image.red[i] = colors[i].red;
 		image.green[i] = colors[i].green;
 		image.blue[i] = colors[i].blue;
-		Red[i] = (unsigned char) image.red[i];
-		Green[i] = (unsigned char) image.green[i];
-		Blue[i] = (unsigned char) image.blue[i];
+		Red[i] = (byte) image.red[i];
+		Green[i] = (byte) image.green[i];
+		Blue[i] = (byte) image.blue[i];
 	}
 	BusyCursor();
 
-	WriteGIF(out_file, image.ximage->data, width, height, Red, Green, Blue, ncolors, 0);
+	WriteGIF(out_file, (byte*)image.ximage->data, width, height, Red, Green, Blue, ncolors, 0);
 	BusyCursor();
 	fclose(out_file);
 	return ;
@@ -1101,7 +1206,7 @@ static int cur_bits = 0;
 
 #define HSIZE  5003		/* 80% occupancy */
 
-typedef unsigned char char_type;
+typedef byte char_type;
 
 
 static int n_bits;		/* number of bits/code */
@@ -1621,6 +1726,8 @@ int fill,num,PtType,FillStyle,BaseType;
                 fill = FILLEDTRI;
         if (num == 1 && PtType == MENU_FILLEDCIRCLE)
                 fill = FILLEDCIRCLE;
+        if (num == 1 && PtType == MENU_FILLEDSPHERE)
+                fill = FILLEDSPHERE;
         if (num > 2)
                 fill = FillStyle;
         if (num == 2 && FillStyle == FILLEDOPAQUE)
@@ -1639,3 +1746,362 @@ int fill,num,PtType,FillStyle,BaseType;
         return fill;
 }
 
+
+void FindWindowSize(win,WinWidth,WinHeight)
+Window win;
+int WinWidth;
+int WinHeight;
+{
+	Window Root;
+	int PosX,PosY;
+	unsigned int BorderWidth,WinDepth;
+	unsigned int Width,Height;
+
+	XGetGeometry(theDisplay,(Window)win,
+                &Root,&PosX,&PosY,&Width,&Height,&BorderWidth,&WinDepth);
+	fprintf(stderr,"%d %d\n",Width,Height);
+	WinWidth = Width;
+	WinHeight = Height;
+	return;
+}
+
+unsigned int GetDrawWidth(Win)
+Window Win;
+{
+	int PosX,PosY;
+        unsigned int WinWidth,WinHeight,BorderWidth,WinDepth;
+        Window Root;
+        XGetGeometry(theDisplay,Win,&Root,&PosX,&PosY,&WinWidth,
+				&WinHeight,&BorderWidth,&WinDepth);
+/*	return WinWidth;*/
+	return DrawingAreaWidth;
+}
+
+unsigned int GetDrawHeight(Win)
+Window Win;
+{
+        int PosX,PosY;
+        unsigned int WinWidth,WinHeight,BorderWidth,WinDepth;
+        Window Root;
+        XGetGeometry(theDisplay,Win,&Root,&PosX,&PosY,&WinWidth,
+                                &WinHeight,&BorderWidth,&WinDepth);
+/*        return WinHeight;*/
+	return DrawingAreaHeight;
+}
+
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+void
+OutputPICT(fname, x, y, width, height, GraphicArea)
+char * fname;
+int x;
+int y;
+int width;
+int height;
+Window GraphicArea;
+{
+        imageInfo image;
+        FILE *out_file;
+        int i, ncolors;
+        int BuffCount;
+        Colormap cmap;
+	char *packed;
+	int oc;
+        int ArrayIndex;
+	byte *PictRed,*PictBlue,*PictGreen;
+        XColor colors[MAX_CELLS];
+        byte *newBuffer;
+        XImage *ximage;
+	int cmaplen,j,row;
+	int lmaxval;
+	int nc;
+
+
+        /* Open Output File */
+        /* At later date this will be made from file selector */
+        BusyCursor();
+        out_file = fopen(fname, "w");
+        /* Grap window to Pixmap */
+        width-=2;
+        ximage = XGetImage(theDisplay, GraphicArea, x, y,(unsigned int) width,(unsigned int) height, AllPlanes, format);
+        image.ximage = ximage;
+
+        cmap = DefaultColormap(theDisplay, theScreen);
+        ncolors = DisplayCells(theDisplay, theScreen);
+        for (i = 0; i < ncolors; i++) {
+                colors[i].pixel = i;
+        }
+        BusyCursor();
+
+        PictRed = (byte *) XmgfMalloc(ncolors * sizeof(byte));
+	assert(PictRed);
+	PictGreen = (byte *) XmgfMalloc(ncolors * sizeof(byte));
+	assert(PictRed);
+	PictBlue = (byte *) XmgfMalloc(ncolors * sizeof(byte));
+	assert(PictRed);
+
+        XQueryColors(theDisplay, cmap, colors, ncolors);
+        for (i = 0; i < ncolors; i++) {
+                PictRed[i] = image.red[i] = colors[i].red;
+                PictGreen[i] = image.green[i] = colors[i].green;
+                PictBlue[i] = image.blue[i] = colors[i].blue;
+        }
+        BusyCursor();
+
+        /* Trasnfer Pixmap to PICT file */
+        newBuffer = (byte *) XmgfMalloc(width * height * 3 * sizeof(byte));
+        assert(newBuffer);
+        BuffCount = 0;
+        BusyCursor();
+        for (i = 0; i < width * height; i++) {
+                ArrayIndex = image.ximage->data[i];
+                newBuffer[BuffCount++] = (byte) image.red[ArrayIndex];
+                newBuffer[BuffCount++] = (byte) image.green[ArrayIndex];
+                newBuffer[BuffCount++] = (byte) image.blue[ArrayIndex];
+        }
+        BusyCursor();
+	WritePictHeader(out_file,height,width+5);
+
+      for (i=0; i<256; i++) { pc2nc[i] = r1[i] = g1[i] = b1[i] = 0; }
+
+      nc = 0;
+      for (i=0; i<256; i++) {
+      /* see if color #i is a duplicate */
+      for (j=0; j<i; j++) {
+        if (PictRed[i] == PictRed[j] && PictGreen[i] == PictGreen[j] &&
+            PictBlue[i] == PictBlue[j]) break;
+      }
+
+      if (j==i) {  /* wasn't found */
+        pc2nc[i] = nc;
+        r1[nc] = PictRed[i];
+        g1[nc] = PictGreen[i];
+        b1[nc] = PictBlue[i];
+        nc++;
+      }
+      else pc2nc[i] = pc2nc[j];
+    }
+
+        cmaplen = 1<<8;   /* 256 */
+
+        /* Write out the colormap. */
+        for (i = 0; i < cmaplen; i++)
+        {
+                putShort(out_file, i);
+
+                  j = MONO(r1[i],b1[i],g1[i]);
+
+                putShort(out_file, j);
+                putShort(out_file, j);
+                putShort(out_file, j);
+        printf("Colormap[%d]: %d %d %d\n",i,j,j,j);
+
+        }
+
+        putRect(out_file, 0, 0, height, width);  /* srcRect */
+        putRect(out_file, 0, 0, height, width);  /* dstRect */
+        putShort(out_file, 0);                        /* Transfer Mode */
+
+	packed = (char*) XmgfMalloc((unsigned)(width+width/MAX_COUNT+1));
+        oc = 0;
+
+        for (row = 0; row < height; row++)
+            oc += putRow(out_file, row, width,newBuffer,packed);
+
+	/* if we wrote an odd number of pixdata bytes, pad */
+/*        if (oc & 1) (void) putc(0, stdout);*/
+
+        putShort(out_file, PICT_EndOfPicture);
+
+        lmaxval = ftell(out_file) - HEADER_SIZE;
+        if (fseek(out_file, (long)HEADER_SIZE, 0) >= 0)
+                putShort(out_file, (short)(lmaxval & 0xffff));
+
+        BusyCursor();
+        fclose(out_file);
+        return ;
+}
+
+/*
+** Using code From ....
+** ppmtopict.c - read a portable pixmap and produce a Macintosh PICT2 file.
+**
+** Copyright (C) 1990 by Ken Yap <ken@cs.rochester.edu>.
+**
+** Permission to use, copy, modify, and distribute this software and its
+** documentation for any purpose and without fee is hereby granted, provided
+** that the above copyright notice appear in all copies and that both that
+** copyright notice and this permission notice appear in supporting
+** documentation.  This software is provided "as is" without express or
+** implied warranty.
+*/
+
+void WritePictHeader(fp,rows,cols)
+FILE *fp;
+int rows;
+int cols;
+{
+        /* write the header */
+        putFill(fp, HEADER_SIZE);
+
+        /* write picSize and picFrame */
+        putShort(fp, 0);
+        putRect(fp, 0, 0, rows, cols);
+
+        /* write version op and version */
+        putShort(fp, PICT_picVersion);
+        putShort(fp, 0x02FF);
+        putShort(fp, PICT_headerOp);
+        putLong(fp, -1L);
+        putFixed(fp, 0, 0);
+        putFixed(fp, 0, 0);
+        putFixed(fp, cols, 0);
+        putFixed(fp, rows, 0);
+        putFill(fp, 4);
+
+        /* seems to be needed by many PICT2 programs */
+        putShort(fp, PICT_clipRgn);
+        putShort(fp, 10);
+        putRect(fp, 0, 0, rows, cols);
+
+        /* write picture */
+        putShort(fp, PICT_BitsRect);
+        putShort(fp, cols | 0x8000);
+        putRect(fp, 0, 0, rows, cols);
+        putShort(fp, 0);        /* pmVersion */
+        putShort(fp, 0);        /* packType */
+        putLong(fp, 0L);        /* packSize */
+        putFixed(fp, 72, 0);    /* hRes */
+        putFixed(fp, 72, 0);    /* vRes */
+        putShort(fp, 0);        /* pixelType */
+        putShort(fp, 8);        /* pixelSize */
+        putShort(fp, 1);        /* cmpCount */
+        putShort(fp, 8);        /* cmpSize */
+        putLong(fp, 0L);        /* planeBytes */
+        putLong(fp, 0L);        /* pmTable */
+        putLong(fp, 0L);        /* pmReserved */
+        putLong(fp, 0L);        /* ctSeed */
+        putShort(fp, 0);        /* ctFlags */
+        putShort(fp,  255);     /* ctSize */
+}
+
+#define MONO(rd,gn,bl) (((rd)*11 + (gn)*16 + (bl)*5) >> 5)
+static void
+putFill(fd, n)
+FILE *fd;
+int n;
+{
+        register int i;
+
+        for (i = 0; i < n; i++)
+                (void) putc(0, fd);
+}
+
+static void
+putShort(fd, i)
+FILE *fd;
+int i;
+{
+        (void) putc((i >> 8) & 0xff, fd);
+        (void) putc(i & 0xff, fd);
+}
+
+
+
+static void
+putLong(fd, i)
+FILE *fd;
+long i;
+{
+        (void) putc((int)((i >> 24) & 0xff), fd);
+        (void) putc(((int)(i >> 16) & 0xff), fd);
+        (void) putc(((int)(i >> 8) & 0xff), fd);
+        (void) putc((int)(i & 0xff), fd);
+}
+
+static void
+putFixed(fd, in, frac)
+FILE *fd;
+int in, frac;
+{
+        putShort(fd, in);
+        putShort(fd, frac);
+}
+
+static void
+putRect(fd, x1, x2, y1, y2)
+FILE *fd;
+int x1, x2, y1, y2;
+{
+        putShort(fd, x1);
+        putShort(fd, x2);
+        putShort(fd, y1);
+        putShort(fd, y2);
+}
+
+#define         runtochar(c)    (257-(c))
+#define         counttochar(c)  ((c)-1)
+#define PPM_EQUAL(p,q) ((p) == (q))
+
+static int
+putRow(fd, row, cols,rowpixels,packed)
+FILE *fd;
+int row, cols;
+byte *rowpixels;
+char *packed;
+{
+        register int i, j;
+	register int bc,oc;
+	byte Rin,Gin,Bin;
+	byte *pP;
+	byte val;
+
+        bc = cols + (cols + MAX_COUNT - 1) / MAX_COUNT;
+        if (bc > 250)
+        {
+                putShort(fd, bc);
+                oc = bc + 2;
+        }
+        else
+        {
+                (void) putc(bc, fd);
+                oc = bc + 1;
+        }
+
+
+        for (i = 0,pP = rowpixels;i < cols;)
+        {
+          if (cols - i > MAX_COUNT)
+          {
+         	(void) putc(127, fd);
+                       for (j = 0; j < MAX_COUNT; j++)
+                        {
+			Rin = (byte)pP[0]; pP++;
+			Gin = (byte)pP[0]; pP++;
+			Bin = (byte)pP[0]; pP++;
+         		val = MONO(Rin,Gin,Bin);
+			(void) putc(val & 0xff, fd);
+                        }
+                        i += MAX_COUNT;
+          }
+          else
+          {
+                        (void) putc(cols-i-1, fd);
+                        for (j = 0; j < cols - i; j++)
+                        {
+			Rin = (byte)pP[0]; pP++;
+			Gin = (byte)pP[0]; pP++;
+			Bin = (byte)pP[0]; pP++;
+         		val = MONO(Rin,Gin,Bin);
+			fprintf(stderr,"%d ",val);
+			(void) putc(val & 0xff, fd);
+                        }
+                        i = cols;
+                }
+        }
+        printf("Row #: %d %d\n",row,cols);
+	return(oc);
+}
+
+
+
