@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
  * README file.
  *
@@ -18,12 +19,21 @@
 #include "VersionInfo.h"
 #include "Disp3DGF.h"
 #include "Init.h"
+#include "XmgfFont.h"
 
 int     OutPost;
 int     OutPostCol;
 int     PostEPS;
 FILE   *PSFilePtr;
 
+#ifndef _NO_PROTO 
+void DetermineColour(int Color,char *ColourString);
+void PSMakeOutLine(XPoint *Pts,int num,int LineWidth);
+#else 
+void DetermineColour();
+void PSMakeOutLine();
+#endif 
+
 
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 /* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
@@ -133,42 +143,12 @@ int     ShadedVal;
   int     i;
   double  PSLineWidth = (double) LineWidth / 2;
   char    ColourString[10];
+  char    HaloedColourString[10];
+  char    FontName[30];
   int     P = DefaultPointSize;
 
-  switch (Color) {
-   case 64:
-    strcpy(ColourString, "0 0 0 srgb");
-    break;
-   case 66:
-    strcpy(ColourString, "0 0 1 srgb");
-    break;
-   case 69:
-    strcpy(ColourString, "0 1 0 srgb");
-    break;
-   case 72:
-    strcpy(ColourString, "0 1 1 srgb");
-    break;
-   case 65:
-    strcpy(ColourString, "1 0 0 srgb");
-    break;
-   case 92:
-    strcpy(ColourString, "1 0 1 srgb");
-    break;
-   case 67:
-    strcpy(ColourString, "1 1 0 srgb");
-    break;
-   case 70:
-    strcpy(ColourString, "1 1 1 srgb");
-    break;
-   case -1:
-    strcpy(ColourString, "0 0 0 srgb");
-    break;
-   default:
-    if (Color > 64)
-      Color = 0;
-    sprintf(ColourString, "%.2f sgy\n", ((double) Color / 64));
-    break;
-  }
+  DetermineColour(Color,ColourString);
+  DetermineColour(HaloedColor,HaloedColourString);
 
   fill = DetermineFillType(fill, num, PtType, FillStyle, BaseType);
 
@@ -302,6 +282,12 @@ int     ShadedVal;
       fprintf(PSFilePtr, "s\n");
       break;
     }
+   case FILLEDSPHERE:{
+      fprintf(PSFilePtr, "%s\n", ColourString);
+      fprintf(PSFilePtr, "%d %d %d %f %f sp\n", (int) Pts[0].x, (int) -Pts[0].y,P,(float)P/4.0,(float)P/8.0);
+      break;
+   }
+
    case FILLEDOPAQUE:{
       fprintf(PSFilePtr, "n\n");
       fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
@@ -371,13 +357,36 @@ int     ShadedVal;
       fprintf(PSFilePtr, "fill\n");
       break;
     }
-   case FILLEDHIDDENLINE:{
+   case HALOEDLINES:{
       fprintf(PSFilePtr, "n\n");
       fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
       for (i = 1; i < num; i++) {
 	fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
       }
       fprintf(PSFilePtr, "clp\n");
+      fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth+HALSIZE);
+      fprintf(PSFilePtr, "%s\n", HaloedColourString);
+/*      fprintf(PSFilePtr, "1.0 sgy\n");*/
+      fprintf(PSFilePtr, "s\n");
+      fprintf(PSFilePtr, "n\n");
+      fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
+      for (i = 1; i < num; i++) {
+	fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
+      }
+      fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
+      fprintf(PSFilePtr, "%s\n", ColourString);
+      fprintf(PSFilePtr, "s\n");
+      PSMakeOutLine(Pts,num,LineWidth);
+      break;
+    }
+  
+   case FILLEDHIDDENLINE:{
+      fprintf(PSFilePtr, "n\n");
+      fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
+      for (i = 1; i < num; i++) {
+        fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
+      }
+      fprintf(PSFilePtr, "clp\n");
       fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
       fprintf(PSFilePtr, ".2 sgy\n");
       fprintf(PSFilePtr, "fill\n");
@@ -385,12 +394,15 @@ int     ShadedVal;
       fprintf(PSFilePtr, "n\n");
       fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
       for (i = 1; i < num; i++) {
-	fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
+        fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
       }
       fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
       fprintf(PSFilePtr, "s\n");
+      PSMakeOutLine(Pts,num,LineWidth);
       break;
     }
+ 
+	
    case COLOUREDLINES:
    case FILLEDCOLOR:{
       fprintf(PSFilePtr, "n\n");
@@ -402,6 +414,7 @@ int     ShadedVal;
       fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
       fprintf(PSFilePtr, "%s\n", ColourString);
       fprintf(PSFilePtr, "fill\n");
+      PSMakeOutLine(Pts,num,LineWidth);
       break;
     }
    case POINTS_ONLY:{
@@ -423,7 +436,16 @@ int     ShadedVal;
    case TEXTLABEL:{
       fprintf(PSFilePtr, "%s\n", ColourString);
       fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
-      fprintf(PSFilePtr, "(%s) /Helvetica printword\n", CurrentString);
+	switch(FontFamily){
+	 case COURIER: strcpy(FontName,"/Courier");break;
+	 case NEWCENSCH: strcpy(FontName,"/NewCenturySchlbk-Roman");break;
+	 case HELVETICA: strcpy(FontName,"/Helvetica");break;
+	 case SYMBOL: strcpy(FontName,"/Symbol");break;
+	 case TIMES: strcpy(FontName,"/Times-Roman");break;
+	 default: strcpy(FontName,"/Helvetica");break;
+	}
+      fprintf(PSFilePtr, "(%s) %d %s printword\n", CurrentString,FontSize,FontName);
+/*      fprintf(PSFilePtr, "(%s) /Helvetica printword\n", CurrentString);*/
       break;
     }
 
@@ -436,6 +458,7 @@ int     ShadedVal;
       fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
       fprintf(PSFilePtr, "%s\n", ColourString);
       fprintf(PSFilePtr, "s\n");
+      PSMakeOutLine(Pts,num,LineWidth);
       break;
     }
   }
@@ -501,8 +524,26 @@ int     Orientation;
   fprintf(PSFilePtr, "/slw {setlinewidth} bind def\n");
   fprintf(PSFilePtr, "/sgy {setgray} bind def\n");
   fprintf(PSFilePtr, "/srgb {setrgbcolor} bind def\n");
-  fprintf(PSFilePtr, "/printword{ gsave findfont\
-			  10 scalefont setfont show grestore } def\n");
+  fprintf(PSFilePtr, "/printword{ gsave findfont exch scalefont setfont show grestore } def\n");
+  /* Define rules for sphere */
+  fprintf(PSFilePtr, "/disk { 0 360 arc } bind def\n");
+  fprintf(PSFilePtr, "/filldisk { disk fill } bind def\n");
+  fprintf(PSFilePtr, "/clipdisk { disk clip newpath } bind def\n");
+#ifdef SPHEREPSGREY 
+  fprintf(PSFilePtr, "/sphere { 1  -.05  0 { \n");
+  fprintf(PSFilePtr, "4 copy dup 1 exch sub  setgray mul filldisk pop \n");
+  fprintf(PSFilePtr, "}for pop pop pop } bind def\n");
+#else
+  fprintf(PSFilePtr, "/sphere { 1  -.05  0 {4 copy dup 1 exch sub \n");
+  fprintf(PSFilePtr, "currentrgbcolor  pop 3 -1 roll setrgbcolor mul\n");
+  fprintf(PSFilePtr, "filldisk pop }for pop pop pop } bind def\n");
+#endif
+
+  fprintf(PSFilePtr, "/DrSphere { gsave 5 2 roll 3 copy clipdisk 5 1 roll \n");
+  fprintf(PSFilePtr, "2 index add  exch 3 index add exch 5 -1 roll 1.4 mul \n");
+  fprintf(PSFilePtr, "sphere pop pop grestore } bind def\n");
+  fprintf(PSFilePtr, "/sp {DrSphere} bind def\n");
+
   if (Orientation == 1) {
     fprintf(PSFilePtr, "0 -842 translate\n");
     fprintf(PSFilePtr, "%d %d translate\n", -PMiX, PMiY);
@@ -526,3 +567,70 @@ EndPostScript()
   PostEPS = 0;
   return ;
 }
+
+void
+PSMakeOutLine(Pts,num,LineWidth)
+XPoint *Pts;
+int num;
+int LineWidth;
+{
+    char ColourString[10];
+    int i;
+    double  PSLineWidth = (double) LineWidth / 2;
+
+    if(OutLine){
+    DetermineColour(OutLineColour,ColourString);
+      fprintf(PSFilePtr, "n\n");
+      fprintf(PSFilePtr, "%d %d m\n", (int) Pts[0].x, (int) -Pts[0].y);
+      for (i = 1; i < num; i++) {
+        fprintf(PSFilePtr, "%d %d l\n", (int) Pts[i].x, (int) -Pts[i].y);
+      }
+      fprintf(PSFilePtr, "clp\n");
+      fprintf(PSFilePtr, "%.2f slw\n", PSLineWidth);
+      fprintf(PSFilePtr, "%s\n", ColourString);
+      fprintf(PSFilePtr, "s\n");
+    }
+}
+
+
+void DetermineColour(Color,ColourString)
+int Color;
+char *ColourString;
+{
+	switch (Color) {
+   	case 64:
+     		strcpy(ColourString, "0 0 0 srgb");
+     		break;
+   	case 66:
+     		strcpy(ColourString, "0 0 1 srgb");
+     		break;
+   	case 69:
+     		strcpy(ColourString, "0 1 0 srgb");
+     		break;
+   	case 72:
+     		strcpy(ColourString, "0 1 1 srgb");
+     		break;
+   	case 65:
+     		strcpy(ColourString, "1 0 0 srgb");
+     		break;
+   	case 92:
+     		strcpy(ColourString, "1 0 1 srgb");
+     		break;
+   	case 67:
+     		strcpy(ColourString, "1 1 0 srgb");
+     		break;
+   	case 70:
+     		strcpy(ColourString, "1 1 1 srgb");
+     		break;
+   	case -1:
+     		strcpy(ColourString, "0 0 0 srgb");
+     		break;
+   	default:
+		 if (Color > 64)
+      		Color = 0;
+    		sprintf(ColourString, "%.2f sgy\n", ((double) Color / 64));
+    		break;
+  	}
+}
+
+
