@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad
  * This file is part of the Xmgf Software.
  * For conditions of distribution and use, see the accompanying README file.
  *
@@ -39,6 +40,7 @@
 #include <X11/StringDefs.h>
 #ifdef MOTIF
 #include <Xm/XmP.h>
+#include <Xm/DrawP.h>
 #endif
 
 #include "RheostatP.h"
