@@ -1,5 +1,6 @@
+static char sccsid[]="@(#)%E% P.Hoad@ee.surrey.ac.uk %I% %M%";
 /*
- * Copyright (C) 1992, 1993, Paul D. Hoad This file is part of the Xmgf
+ * Copyright (C) 1992, 1993, 1994, Paul D. Hoad This file is part of the Xmgf
  * Software. For conditions of distribution and use, see the accompanying
  * README file.
  *
@@ -8,6 +9,10 @@
  * Ext 2753 Email: P.Hoad@ee.surrey.ac.uk
  */
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <X11/Intrinsic.h>
+#include <X11/Shell.h>
+#include <X11/keysym.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
 #include <Xm/MainW.h>
@@ -28,6 +33,8 @@
 #include "AutoCommand.h"
 #include "BackDrop.h"
 #include "XmgfKeyboard.h"
+#include "XmgfEvent.h"
+#include "Read3DSet.h"
 
 #include "BITMAPS/WarningBits.xbm"
 #include "BITMAPS/BackForth.xbm"
@@ -63,8 +70,8 @@ int      	defOX, defOY, defOZ, defTheta, defPhi, defGamma;
 int      	WindowOpen, LoadOpen, SaveOpen, AppendOpen;
 appresStruct    appres;
 XtAppContext 	appContext;
-int Midx = 256;
-int Midy = 256;
+int Midx;
+int Midy;
 Cursor SpinCursor, LeftRight, UpDown, BackForthCursor;
 Cursor CurTheta, CurPhi, CurGamma;
 Cursor Busy1, Busy2, Busy3, Busy4, Busy5, Busy6, Busy7, Busy8, Busy9;
@@ -72,7 +79,6 @@ Cursor DefaultCursor;
 XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
 int viewtype = 0;
 
-
 #ifndef _NO_PROTO
 Pixmap *CreateDefaultPixmap(char *bits, int width, int height);
 Cursor MakeXmgfCursor(char *bits, int width, int height, int x_hot, int y_hot, char *CurMask);
@@ -81,7 +87,6 @@ void SetUpCursors(void);
 Pixmap *CreateDefaultPixmap();
 Cursor MakeXmgfCursor();
 void SetUpCursors();
-
 #endif
 
 XtResource application_resources[] = {
@@ -96,7 +101,7 @@ XtResource application_resources[] = {
 	{"xmgfsmallheight", "XmgfSmallHeight", XtRInt, sizeof(int),
 	XtOffset(appresPtr, XmgfSmallHeight), XtRImmediate, (XtPointer) 800},
 	{"xmgfsmalliconwidth", "XmgfSmallIconWidth", XtRInt, sizeof(int),
-	XtOffset(appresPtr, XmgfSmallIconWidth), XtRImmediate, (XtPointer) 690},
+	XtOffset(appresPtr, XmgfSmallIconWidth), XtRImmediate, (XtPointer) 710},
 	{"lineButtColor", "LineButtColor", XtRPixel, sizeof(Pixel),
 	XtOffset(appresPtr, LineButtColor), XtRString, "#00F"},
 	{"lineThickColor", "LineThickColor", XtRPixel, sizeof(Pixel),
@@ -125,6 +130,8 @@ XtResource application_resources[] = {
 	XtOffset(appresPtr, GZipCommand), XtRImmediate, ""},
 	{"removecommand", "RemoveCommand", XtRString, sizeof(String),
 	XtOffset(appresPtr, RemoveCommand), XtRImmediate, ""},
+	{"copycommand", "CopyCommand", XtRString, sizeof(String),
+	XtOffset(appresPtr, CopyCommand), XtRImmediate, ""},
 	{"GFDir", "GFDir", XtRString, sizeof(String),
 	XtOffset(appresPtr, GFDir), XtRImmediate, "."},
 	{"PGMDir", "PGMDir", XtRString, sizeof(String),
@@ -155,6 +162,10 @@ XtResource application_resources[] = {
 	XtOffset(appresPtr, MIFDir), XtRImmediate, "."},
 	{"ANIMDir", "ANIMDir", XtRString, sizeof(String),
 	XtOffset(appresPtr, ANIMDir), XtRImmediate, "."},
+	{"PCXDir", "PCXDir", XtRString, sizeof(String),
+	XtOffset(appresPtr, PCXDir), XtRImmediate, "."},
+	{"PICTDir", "PICTDir", XtRString, sizeof(String),
+	XtOffset(appresPtr, PICTDir), XtRImmediate, "."},
 	{"IMPORTDir", "IMPORTDir", XtRString, sizeof(String),
 	XtOffset(appresPtr, IMPORTDir), XtRImmediate, "."},
 	{"CMAPDir", "CMAPDir", XtRString, sizeof(String),
@@ -181,6 +192,12 @@ XtResource application_resources[] = {
 	XtOffset(appresPtr, XMgfRelease), XtRImmediate, "3"},
 	{"XMgfRelDate", "XMgfRelDate", XtRString, sizeof(String),
 	XtOffset(appresPtr, XMgfRelDate), XtRImmediate, "23/8/93"},
+	{"HelpFile", "HelpFile", XtRString, sizeof(String),
+	XtOffset(appresPtr, HelpFile), XtRImmediate, "Xmgf.html"},
+	{"HelpDir", "HelpDir", XtRString, sizeof(String),
+	XtOffset(appresPtr, HelpDir), XtRImmediate, "/usr/local/lib/Xmgf/html"},
+	{"MosaicCmd", "MosaicCmd", XtRString, sizeof(String),
+	XtOffset(appresPtr, MosaicCmd), XtRImmediate, "xmosaic"},
 };
 
 XrmOptionDescRec options[] = {
@@ -219,6 +236,8 @@ char **argv;
 
 	/* Set Up as much as Possible */
 	gfSets.ListXmgfSets = ConsLL();
+	DrawingAreaWidth=530;
+	DrawingAreaHeight=530;
 	InitDraw();
 	NameList = ConsLL();
 	ColorNameList = ConsLL();
@@ -231,9 +250,18 @@ char **argv;
 	AppendOpen = 0;
 
 	/* SetUp Waring Symbols */
-	Warning_image = CreateDefaultImage(WarningBits_bits, WarningBits_width, WarningBits_height);
+	Warning_image = CreateDefaultImage(WarningBits_bits, WarningBits_width, 
+				           WarningBits_height);
 	XmInstallImage(Warning_image, "Warning_image");
 
+	/* 
+        * Still in some doubt what I need this for but I have noticed
+	*  more and more applications are placing this in now 
+	*  if someone can give me some pointers I will be very pleased
+        *  Something to do with keyboards ? 
+        *  XtSetLanguageProc(NULL, NULL, NULL); 
+        */
+	   
 
 	/* |//////////////   Create The Application   \\\\\\\\\\\\\\\\\| */
 	toplevel = XtAppInitialize(&appContext, "XMgf", (XrmOptionDescList) options,
@@ -241,25 +269,25 @@ char **argv;
 	XtAppAddActions(appContext, XmgfActsTable, XtNumber(XmgfActsTable));
 	XtGetApplicationResources(toplevel, &appres, application_resources,
 		XtNumber(application_resources), NULL, 0);
+	XtOverrideTranslations(toplevel,XtParseTranslationTable(XmgfTranslation));
 	/* |//////////////   Parse The Command Line   \\\\\\\\\\\\\\\\\| */
+
+/* This Part is for using Editres with Xmgf */
 {
-/*
+#if (XtSpecificationRelease > 4) && defined(I_WANT_EDITRES)
 #include <X11/Xmu/Editres.h>
         XtAddEventHandler(toplevel, (EventMask) 0, True,
                 _XEditResCheckMessages, NULL);
-*/
+#endif
 }
+
 	ReadInit();
 	ParseCommandLine(argc, argv);
 	InitViewScale();
 
 	/* |//////////////   Open the main window  \\\\\\\\\\\\\\\\\| */
 
-	if(MAINPANEL){
-	XtSetArg(args[0], XtNwidth, (int) appres.XmgfWidth);
-	XtSetArg(args[1], XtNheight, (int) appres.XmgfHeight);
-	}
-	else if (!ICONPANEL){
+	if (!ICONPANEL){
 	XtSetArg(args[0], XtNwidth, (int) appres.XmgfSmallWidth);
 	XtSetArg(args[1], XtNheight, (int) appres.XmgfSmallHeight);
 	}
@@ -267,8 +295,8 @@ char **argv;
 	XtSetArg(args[0], XtNwidth, (int) appres.XmgfSmallIconWidth);
 	XtSetArg(args[1], XtNheight, (int) appres.XmgfSmallHeight);
 	}
-
-	main_window = XmCreateMainWindow(toplevel, "main1", args, 2);
+	XtSetArg(args[2], XmNallowShellResize, True);  
+	main_window = XmCreateMainWindow(toplevel, "main1", args, 3);
 	XtManageChild(main_window);
 
 	/* |//////////////   Find the Display CMap \\\\\\\\\\\\\\\\\| */
@@ -294,6 +322,8 @@ char **argv;
 	while (!XtIsRealized(Orthdr)) {
 	}			/* Wait Until drawing window open is realised */
 	WindowOpen = 1;
+	XtAddEventHandler(toplevel, StructureNotifyMask, False,
+                (XtEventHandler) XmgfWindowEvent, (caddr_t) NULL);
 	InitFiles();
 	ResetScaleNoRedraw();
 	ChangeAngleLab();
@@ -373,6 +403,8 @@ char *curmask;
 	return NewCursor;
 }
 
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 void
 BusyCursor()
 {
@@ -382,70 +414,40 @@ BusyCursor()
 	CurrentCursor = Busy1;
 
 	switch (BusyFlag) {
-	    case 1:
-		CurrentCursor = Busy9;
-		break;
-	    case 2:
-		CurrentCursor = Busy8;
-		break;
-	    case 3:
-		CurrentCursor = Busy7;
-		break;
-	    case 4:
-		CurrentCursor = Busy6;
-		break;
-	    case 5:
-		CurrentCursor = Busy5;
-		break;
-	    case 6:
-		CurrentCursor = Busy4;
-		break;
-	    case 7:
-		CurrentCursor = Busy3;
-		break;
-	    case 8:
-		CurrentCursor = Busy2;
-		break;
-	    case 9:
-		CurrentCursor = Busy1;
-		break;
+	    case 1: CurrentCursor = Busy9; break;
+	    case 2: CurrentCursor = Busy8; break;
+	    case 3: CurrentCursor = Busy7; break;
+	    case 4: CurrentCursor = Busy6; break;
+	    case 5: CurrentCursor = Busy5; break;
+	    case 6: CurrentCursor = Busy4; break;
+	    case 7: CurrentCursor = Busy3; break;
+	    case 8: CurrentCursor = Busy2; break; 
+	    case 9: CurrentCursor = Busy1; break;
 	}
 	BusyFlag++;
 	if (BusyFlag > 9)
 		BusyFlag = 1;
 	XDefineCursor(theDisplay, XtWindow(toplevel), CurrentCursor);
 	XDefineCursor(theDisplay, w_Orth, CurrentCursor);
-#ifdef MAIN_PANEL
-	XDefineCursor(theDisplay, XtWindow(OY), CurrentCursor);
-	XDefineCursor(theDisplay, XtWindow(OX), CurrentCursor);
-	XDefineCursor(theDisplay, XtWindow(OZ), CurrentCursor);
-	XDefineCursor(theDisplay, XtWindow(w_Theta), CurrentCursor);
-	XDefineCursor(theDisplay, XtWindow(w_Gamma), CurrentCursor);
-	XDefineCursor(theDisplay, XtWindow(w_Phi), CurrentCursor);
-#endif
 	if (LoadOpen)
 		XDefineCursor(theDisplay, XtWindow(FileSelector), CurrentCursor);
 
 }
 
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 void
 DoneCursor()
 {
 	XDefineCursor(theDisplay, XtWindow(toplevel), DefaultCursor);
 	XDefineCursor(theDisplay, w_Orth, SpinCursor);
-#ifdef MAIN_PANEL
-	XDefineCursor(theDisplay, XtWindow(OY), UpDown);
-	XDefineCursor(theDisplay, XtWindow(OX), LeftRight);
-	XDefineCursor(theDisplay, XtWindow(OZ), BackForthCursor);
-	XDefineCursor(theDisplay, XtWindow(w_Theta), CurGamma);
-	XDefineCursor(theDisplay, XtWindow(w_Gamma), CurTheta);
-	XDefineCursor(theDisplay, XtWindow(w_Phi), CurPhi);
-#endif
 	if (LoadOpen)
 		XDefineCursor(theDisplay, XtWindow(FileSelector), DefaultCursor);
 
 }
 
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
 void
 SetUpCursors()
 {
@@ -484,3 +486,5 @@ SetUpCursors()
 		HourGlass9_x_hot, HourGlass9_y_hot, HourGlass9Mask_bits);
 
 }
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
+/* |/////////////////////////////oOo\\\\\\\\\\\\\\\\\\\\\\\\\\\| */
